import pytest
from odoo.exceptions import UserError, ValidationError
from odoo.tests.common import TransactionCase
from odoo.addons.wsmail_ypf.utils.mail_wrapper import DummyWrapper

import mock
from pytest_mock import mocker
from suds.client import Client
import urllib, os
import json
import sys
import base64

class TestWsMail:


    def test_mail_server_connection(self, MSERV, mocker):
        serv = MSERV.create({
            'name': 'Test',
            'server_type': 'ws',
            'sequence': 10,
            'smtp_host': 'http://www.dneonline.com/calculator.asmx?WSDL',
            'smtp_encryption': 'ssl',
            'smtp_pass': 'ABCD-EFG-HIJ',
        })
        serv.onchange_server_type()
        assert serv.smtp_encryption == 'none'

        with pytest.raises(UserError, match=r".*Connection Test Succeeded!*"):
            serv.test_smtp_connection()

        serv.write({
            'smtp_host': 'http://www.fakeurl.com?wsdl',
        })
        with pytest.raises(UserError, match=r".*Connection Test Failed!*"):
            serv.test_smtp_connection()

    def create_attachment(self, ATTACH):
        attach1 = ATTACH.create({
            'name': 'Attach1',
            'datas_fname': 'Attach1',
            'datas': 'TWFpbCBXZWIgU2VydmljZSBUZXN0',
        })
        return attach1

    def get_mail_formatted_cases(self, ATTACH):
        attach = self.create_attachment(ATTACH)
        mail_formatted_lst = [{
            'ws_method': 'SendFormattedEmail',
            'subject': 'SendFormattedEmail',
            'email_from': 'admin@yourcompany.example.com',
            'email_to': 'receiver@yourcompany.example.com',
            'attachment_ids': [(6, False, attach.ids)],
            'body': 'TEST :D',
        }, {
            'ws_method': 'SendFormattedEmailWithCC',
            'subject': 'SendFormattedEmailWithCC',
            'email_from': 'admin@yourcompany.example.com',
            'email_to': 'receiver@yourcompany.example.com',
            'email_cc': 'receivercc@yourcompany.example.com',
            'attachment_ids': [(6, False, attach.ids)],
            'body': 'TEST :D'
        }, {
            'ws_method': 'SendFormattedEmailNA',
            'subject': 'SendFormattedEmailNA',
            'email_from': 'admin@yourcompany.example.com',
            'email_to': 'receiver@yourcompany.example.com',
            'body': 'TEST :D'
        },{
            'ws_method': 'SendFormattedEmailNAWithCC',
            'subject': 'SendFormattedEmailNAWithCC',
            'email_from': 'admin@yourcompany.example.com',
            'email_to': 'receiver@yourcompany.example.com',
            'email_cc': 'receivercc@yourcompany.example.com',
            'body': 'TEST :D'
        },{
            'ws_method': 'SendFormattedEmailNAWithCCAndBCC',
            'subject': 'SendFormattedEmailNAWithCCAndBCC',
            'email_from': 'admin@yourcompany.example.com',
            'email_to': 'receiver@yourcompany.example.com',
            'email_cc': 'receivercc@yourcompany.example.com',
            'email_bcc': 'bccreceiver@yourcompany.example.com',
            'body': 'TEST :D'
        }]
        return mail_formatted_lst


    def get_mail_cases(self, ATTACH):
        attach = self.create_attachment(ATTACH)
        mail_lst = [{
            'ws_method': 'SendEmailNA',
            'subject': 'SendEmailNA',
            'email_from': 'admin@yourcompany.example.com',
            'email_to': 'receiver@yourcompany.example.com',
            'body': 'TEST :D'
        }, {
            'ws_method': 'SendEmailNAWithCC',
            'subject': 'SendEmailNAWithCC',
            'email_from': 'admin@yourcompany.example.com',
            'email_to': 'receiver@yourcompany.example.com',
            'email_cc': 'receivercc@yourcompany.example.com',
            'body': 'TEST :D'
        }, {
            'ws_method': 'SendEmail',
            'subject': 'SendEmail',
            'email_from': 'admin@yourcompany.example.com',
            'email_to': 'receiver@yourcompany.example.com',
            'attachment_ids': [(6, False, attach.ids)],
            'body': 'TEST :D'
        }, {
            'ws_method': 'SendEmailWithCC',
            'subject': 'SendEmailWithCC',
            'email_from': 'admin@yourcompany.example.com',
            'email_to': 'receiver@yourcompany.example.com',
            'email_cc': 'receivercc@yourcompany.example.com',
            'attachment_ids': [(6, False, attach.ids)],
            'body': 'TEST :D'
        }, {
            'ws_method': 'SendEmailWithCCAndBCC',
            'subject': 'SendEmailWithCCAndBCC',
            'email_from': 'admin@yourcompany.example.com',
            'email_to': 'receiver@yourcompany.example.com',
            'email_cc': 'receivercc@yourcompany.example.com',
            'email_bcc': 'bccreceiver@yourcompany.example.com',
            'attachment_ids': [(6, False, attach.ids)],
            'body': 'TEST :D'
        }]
        return mail_lst

    def get_client(self, test=False):
        ref_path = sys.path[0] + '/wsmail_ypf/tests/data/SendEmail_prod.xml'
        if test:
            ref_path = sys.path[0] + '/wsmail_ypf/tests/data/SendEmail_test.xml'
        path_url = urllib.request.pathname2url(ref_path)
        url = urllib.parse.urljoin('file:', path_url)
        client = Client(url)
        dummy_ctx = {
            'smtp_pass': 'ABCD-EFG-HIJ'
        }
        return DummyWrapper(client, ctx=dummy_ctx)


    def get_methods(self):
        path  = sys.path[0] + '/wsmail_ypf/tests/data/wsdl_methods.json'
        f = open(path, 'r')
        data = f.read()
        obj = json.loads(data)
        return obj

    def get_msg_id(self):
        return "<234957895099899.1578588312.001944780349731-openerp-private@berto>"

    def test_methods(self, MSERV):
        client = self.get_client()
        methods = MSERV.get_ws_methods(client)
        ref_meth = self.get_methods()
        for mth_key, mth_val in methods.items():
            assert mth_key in ref_meth
            values = ref_meth.get(mth_key, False)
            assert len(values) == len(mth_val)
            assert not len(set(values).difference(set(mth_val)))
        return True

    def build_mail(self, MSERV, ATTACH, vals):
        ref_vals = vals.copy()
        email_from = ref_vals.pop('email_from')
        email_to = ref_vals.pop('email_to')
        subject = ref_vals.pop('subject')
        body = ref_vals.pop('body')
        ws_method = ref_vals.pop('ws_method')
        subtype = 'plain'
        if 'Formatted' in ws_method:
            subtype = 'html'
        att_vals = ref_vals.pop('attachment_ids', [])
        attachments = ATTACH
        for att_lst in att_vals:
            attachments += ATTACH.browse(att_lst[2])
        attachments = [
            (a['datas_fname'], base64.b64decode(
                a['datas']), a['mimetype'])
                for a in attachments.sudo().read(
                    ['datas_fname', 'datas', 'mimetype']
                ) if a['datas'] is not False]
        ref_vals.update({
            'attachments': attachments,
            'subtype': subtype
        })
        msg = MSERV.build_email(
                email_from, email_to, subject, body, **ref_vals)

        return msg

    def test_base_send_mail(self, MAIL, MSERV, ATTACH, mocker):
        serv = MSERV.create({
            'name': 'Test',
            'server_type': 'ws',
            'sequence': 1,
            'smtp_host': 'http://www.dneonline.com/calculator.asmx?WSDL',
            'smtp_encryption': 'ssl',
            'smtp_pass': 'ABCD-EFG-HIJ',
        })
        serv.onchange_server_type()
        #Mocks
        m_conn = mocker.patch.object(MSERV.__class__, 'connect')
        m_conn.return_value = self.get_client(test=True)

        m_ws = mocker.patch.object(MSERV.__class__, 'send_ws_mail')
        m_ws.return_value = self.get_msg_id()

        spy_ws_meth = mocker.spy(MSERV.__class__, 'get_ws_method')

        mail_formatted_lst = self.get_mail_formatted_cases(ATTACH)
        #Formatted mails:
        for mail_vals in mail_formatted_lst:
            method = mail_vals.pop('ws_method')
            if method == 'SendFormattedEmailNAWithCCAndBCC':
                continue
            mail = MAIL.create(mail_vals)
            mail.send()
            assert spy_ws_meth.spy_return == method


        #Create Custom Mails:
        mail_formatted_lst = self.get_mail_formatted_cases(ATTACH)
        #Formatted mails:
        for mail_vals in mail_formatted_lst:
            method = mail_vals.get('ws_method')
            msg = self.build_mail(MSERV, ATTACH, mail_vals)
            MSERV.send_email(msg, mail_server_id=serv.id)
            assert spy_ws_meth.spy_return == method

        mail_lst = self.get_mail_cases(ATTACH)
        for mail_vals in mail_lst:
            method = mail_vals.get('ws_method')
            msg = self.build_mail(MSERV, ATTACH, mail_vals)
            MSERV.send_email(msg, mail_server_id=serv.id)
            assert spy_ws_meth.spy_return == method

        return True
