##############################################################################
#   Copyright (c) 2018 Eynes/E-MIPS (www.eynes.com.ar)
#   License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).
##############################################################################


import base64
from email.mime.multipart import MIMEMultipart
from suds.client import Client


class DummyWrapper:
    def __init__(self, obj_to_wrapp, ctx={}):
        self.ctx = ctx
        self.origin_obj = obj_to_wrapp

    def __getitem__(self, key):
        return self.origin_obj[key]

    def __setitem__(self, key, value):
        res = self.origin_obj[key] = value
        return res

    def dummy(self):
        return True

    def __delitem__(self, name):
        del self.origin_obj[name]

    def __getattribute__(self, name):
        attr = False
        try:
            attr = super().__getattribute__(name)
        except AttributeError as e:
            attr = getattr(self.origin_obj, name, False)
            if not attr:
                attr = self.dummy
        return attr


class MIMEMailWrapper:
    def __init__(self, vals, msg_obj):
        for key, value in vals.items():
            setattr(self, key, value)
        self._msg_obj = msg_obj

    def __getitem__(self, key):
        return self._msg_obj[key]

    def __setitem__(self, key, value):
        res = self._msg_obj[key] = value
        return res

    def __delitem__(self, name):
        del self._msg_obj[name]

    def __getattribute__(self, name):
        attr = False
        try:
            attr = super().__getattribute__(name)
        except AttributeError as e:
            attr = getattr(self._msg_obj, name, False)
            if not attr:
                raise AttributeError("Object has no attribute '%s'")%(name)
        return attr


    def get_mime_msg(self):
        return self._msg_obj

    def get_msg_vals(self):
        all_attrs = [
            i for i in self.__dict__.keys()\
            if i[:1] != '_'
        ]
        res_vals = {}
        for attr in all_attrs:
            value = self.__dict__[attr]
            if value or value is not None:
                res_vals[attr] = value
        return res_vals
